@@ -52,28 +52,34 @@ package com.openexchange.filestore.sproxyd;
 import static com.openexchange.filestore.utils.PropertyNameBuilder.optIntProperty;
 import static com.openexchange.filestore.utils.PropertyNameBuilder.requireProperty;
 import static com.openexchange.java.Autoboxing.I;
-import static com.openexchange.osgi.Tools.requireService;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.LinkedList;
 import java.util.List;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
+import java.util.concurrent.Callable;
+import java.util.concurrent.ExecutionException;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 import org.apache.http.client.HttpClient;
 import org.apache.http.client.utils.URIBuilder;
+import com.google.common.cache.Cache;
+import com.google.common.cache.CacheBuilder;
+import com.google.common.cache.RemovalListener;
+import com.google.common.cache.RemovalNotification;
+import com.google.common.util.concurrent.UncheckedExecutionException;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.Interests;
+import com.openexchange.config.Reloadables;
 import com.openexchange.configuration.ConfigurationExceptionCodes;
 import com.openexchange.database.DatabaseService;
 import com.openexchange.exception.OXException;
 import com.openexchange.filestore.DatabaseAccess;
 import com.openexchange.filestore.DatabaseAccessService;
-import com.openexchange.filestore.FileStorage;
 import com.openexchange.filestore.FileStorageInfo;
 import com.openexchange.filestore.FileStorageInfoService;
 import com.openexchange.filestore.FileStorageProvider;
 import com.openexchange.filestore.FileStorages;
+import com.openexchange.filestore.InterestsAware;
 import com.openexchange.filestore.sproxyd.chunkstorage.ChunkStorage;
 import com.openexchange.filestore.sproxyd.chunkstorage.RdbChunkStorage;
 import com.openexchange.filestore.sproxyd.impl.EndpointPool;
@@ -82,6 +88,7 @@ import com.openexchange.filestore.sproxyd.impl.SproxydConfig;
 import com.openexchange.filestore.utils.DefaultDatabaseAccess;
 import com.openexchange.filestore.utils.PropertyNameBuilder;
 import com.openexchange.java.Strings;
+import com.openexchange.metrics.MetricService;
 import com.openexchange.rest.client.httpclient.HttpClients;
 import com.openexchange.rest.client.httpclient.HttpClients.ClientConfig;
 import com.openexchange.server.ServiceLookup;
@@ -92,23 +99,23 @@ import com.openexchange.timer.TimerService;
  *
  * @author <a href="mailto:tobias.friedrich@open-xchange.com">Tobias Friedrich</a>
  */
-public class SproxydFileStorageFactory implements FileStorageProvider {
+public class SproxydFileStorageFactory implements FileStorageProvider, InterestsAware {
 
     private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(SproxydFileStorageFactory.class);
 
-    /**
-     * The URI scheme identifying sproxyd file storages.
-     */
+    /** The URI scheme identifying sproxyd file storages. */
     private static final String SPROXYD_SCHEME = "sproxyd";
 
-    /**
-     * The file storage's ranking compared to other sharing the same URL scheme.
-     */
+    /** The file storage's ranking compared to other sharing the same URL scheme. */
     private static final int RANKING = 547;
 
+    // -------------------------------------------------------------------------------------------------------------------------------------
+
+    /** The service look-up */
     private final ServiceLookup services;
-    private final ConcurrentMap<URI, SproxydFileStorage> storages; // local cache may be removed with v7.8.0
-    private final ConcurrentMap<String, SproxydConfig> sproxydConfigs;
+
+    /** The cache holding initialized end-point pools */
+    private final Cache<String, SproxydConfig> sproxydConfigCache;
 
     /**
      * Initializes a new {@link SproxydFileStorageFactory}.
@@ -118,51 +125,56 @@ public class SproxydFileStorageFactory implements FileStorageProvider {
     public SproxydFileStorageFactory(ServiceLookup services) {
         super();
         this.services = services;
-        this.storages = new ConcurrentHashMap<URI, SproxydFileStorage>();
-        this.sproxydConfigs = new ConcurrentHashMap<String, SproxydConfig>();
+        sproxydConfigCache = CacheBuilder.newBuilder()
+            .weakValues()
+            .removalListener(new RemovalListener<String, SproxydConfig>() {
+
+                @Override
+                public void onRemoval(RemovalNotification<String, SproxydConfig> notification) {
+                    notification.getValue().getEndpointPool().close();
+                    HttpClients.shutDown(notification.getValue().getHttpClient());
+                }
+            })
+            .build();
     }
 
+    @Override
+    public Interests getInterests() {
+        return Reloadables.interestsForProperties("com.openexchange.filestore.sproxyd.*");
+    }
 
     @Override
     public SproxydFileStorage getFileStorage(URI uri) throws OXException {
-        SproxydFileStorage storage = storages.get(uri);
-        if (null == storage) {
-            LOG.debug("Initializing sproxyd file storage for {}", uri);
-            /*
-             * extract context and user from URI path
-             */
-            String filestoreID = extractFilestoreID(uri);
-            ExtractionResult extractionResult = extractFrom(uri);
+        LOG.debug("Initializing sproxyd file storage for {}", uri);
+        /*
+         * extract context and user from URI path
+         */
+        String filestoreID = extractFilestoreID(uri);
+        ExtractionResult extractionResult = extractFrom(uri);
 
-            DatabaseAccess databaseAccess;
-            int contextId;
-            int userId;
-            if (extractionResult.hasContextUserAssociation()) {
-                contextId = extractionResult.getContextId();
-                userId = extractionResult.getUserId();
-                databaseAccess = new DefaultDatabaseAccess(userId, contextId, services.getService(DatabaseService.class));
-                LOG.debug("Using \"{}\" as filestore ID, context ID of filestore is \"{}\", user ID is \"{}\".", filestoreID, I(contextId), I(userId));
-            } else {
-                contextId = 0;
-                userId = 0;
-                databaseAccess = lookUpDatabaseAccess(uri, extractionResult.getPrefix());
-            }
-
-            // Ensure required tables do exist
-            databaseAccess.createIfAbsent(RdbChunkStorage.getRequiredTables());
-
-            /*
-             * initialize file storage using dedicated client & chunk storage
-             */
-            SproxydClient client = initClient(filestoreID, extractionResult.getPrefix());
-            ChunkStorage chunkStorage = new RdbChunkStorage(databaseAccess, contextId, userId);
-            SproxydFileStorage newStorage = new SproxydFileStorage(client, chunkStorage);
-            storage = storages.putIfAbsent(uri, newStorage);
-            if (null == storage) {
-                storage = newStorage;
-            }
+        DatabaseAccess databaseAccess;
+        int contextId;
+        int userId;
+        if (extractionResult.hasContextUserAssociation()) {
+            contextId = extractionResult.getContextId();
+            userId = extractionResult.getUserId();
+            databaseAccess = new DefaultDatabaseAccess(userId, contextId, services.getService(DatabaseService.class));
+            LOG.debug("Using \"{}\" as filestore ID, context ID of filestore is \"{}\", user ID is \"{}\".", filestoreID, I(contextId), I(userId));
+        } else {
+            contextId = 0;
+            userId = 0;
+            databaseAccess = lookUpDatabaseAccess(uri, extractionResult.getRawPrefix());
         }
-        return storage;
+
+        // Ensure required tables do exist
+        databaseAccess.createIfAbsent(RdbChunkStorage.getRequiredTables());
+
+        /*
+         * initialize file storage using dedicated client & chunk storage
+         */
+        SproxydClient client = initClient(filestoreID, extractionResult.getPrefix());
+        ChunkStorage chunkStorage = new RdbChunkStorage(databaseAccess, contextId, userId);
+        return new SproxydFileStorage(uri, client, chunkStorage);
     }
 
     static DatabaseAccess lookUpDatabaseAccess(URI uri, String prefix) throws OXException {
@@ -178,11 +190,6 @@ public class SproxydFileStorageFactory implements FileStorageProvider {
         return databaseAccess;
     }
 
-    @Override
-    public FileStorage getInternalFileStorage(URI uri) throws OXException {
-        return getFileStorage(uri);
-    }
-
     @Override
     public boolean supports(URI uri) {
          return null != uri && SPROXYD_SCHEME.equalsIgnoreCase(uri.getScheme());
@@ -242,7 +249,7 @@ public class SproxydFileStorageFactory implements FileStorageProvider {
         }
 
         // Any path that serves as prefix; e.g. "photos"
-        return new ExtractionResult(sanitizePathForPrefix(path, uri));
+        return new ExtractionResult(sanitizePathForPrefix(path, uri), path);
     }
 
     private static String sanitizePathForPrefix(String path, URI uri) {
@@ -277,28 +284,41 @@ public class SproxydFileStorageFactory implements FileStorageProvider {
      * @return The client
      */
     private SproxydClient initClient(String filestoreID, String prefix) throws OXException {
-        SproxydConfig sproxydConfig = sproxydConfigs.get(filestoreID);
+        SproxydConfig sproxydConfig = initSproxydConfig(filestoreID);
+        return new SproxydClient(sproxydConfig, prefix);
+    }
+
+    /**
+     * Gets the HTTP client and endpoint pool for a configured denoted sproxyd filestore.
+     *
+     * @param filestoreID The filestore ID
+     * @return The endpoint pool
+     * @throws OXException If endpoint pool cannot be initialized
+     */
+    private SproxydConfig initSproxydConfig(String filestoreID) throws OXException {
+        SproxydConfig sproxydConfig = sproxydConfigCache.getIfPresent(filestoreID);
         if (sproxydConfig == null) {
-            SproxydConfig newSproxydConfig = initSproxydConfig(filestoreID);
-            sproxydConfig = sproxydConfigs.putIfAbsent(filestoreID, newSproxydConfig);
-            if (sproxydConfig == null) {
-                sproxydConfig = newSproxydConfig;
-            } else {
-                newSproxydConfig.getEndpointPool().close();
+            try {
+                sproxydConfig = sproxydConfigCache.get(filestoreID, new SproxydConfigLoaderCallable(filestoreID, this));
+            } catch (ExecutionException | UncheckedExecutionException e) {
+                Throwable cause = e.getCause();
+                if (cause instanceof OXException) {
+                    throw (OXException) cause;
+                }
+                throw OXException.general("Failed initializing S3 client.", cause);
             }
         }
-
-        return new SproxydClient(sproxydConfig, prefix);
+        return sproxydConfig;
     }
 
     /**
      * Initializes a new HTTP client and endpoint pool for a configured sproxyd filestore.
      *
      * @param filestoreID The filestore ID
-     * @return The configured items
-     * @throws OXException
+     * @return The new endpoint pool
+     * @throws OXException If endpoint pool cannot be initialized
      */
-    private SproxydConfig initSproxydConfig(String filestoreID) throws OXException {
+    SproxydConfig doInitSproxydConfig(String filestoreID) throws OXException {
         ConfigurationService configService = services.getService(ConfigurationService.class);
         PropertyNameBuilder nameBuilder = new PropertyNameBuilder("com.openexchange.filestore.sproxyd.");
         // End-point configuration
@@ -345,12 +365,13 @@ public class SproxydFileStorageFactory implements FileStorageProvider {
             throw ConfigurationExceptionCodes.INVALID_CONFIGURATION.create("Invalid value for 'com.openexchange.filestore.sproxyd." + filestoreID + ".hosts': " + hosts);
         }
 
-        HttpClient httpClient = HttpClients.getHttpClient(ClientConfig.newInstance()
+        MetricService metrics = services.getServiceSafe(MetricService.class);
+        HttpClient httpClient = HttpClients.getHttpClient(ClientConfig.newInstance("sproxyd-" + filestoreID)
             .setMaxTotalConnections(maxConnections)
             .setMaxConnectionsPerRoute(maxConnectionsPerHost)
             .setConnectionTimeout(connectionTimeout)
             .setSocketReadTimeout(socketReadTimeout));
-        EndpointPool endpointPool = new EndpointPool(filestoreID, urls, httpClient, heartbeatInterval, requireService(TimerService.class, services));
+        EndpointPool endpointPool = new EndpointPool(filestoreID, urls, httpClient, heartbeatInterval, services.getServiceSafe(TimerService.class), metrics);
         return new SproxydConfig(httpClient, endpointPool);
     }
 
@@ -375,4 +396,23 @@ public class SproxydFileStorageFactory implements FileStorageProvider {
         return authority;
     }
 
+    // -------------------------------------------------------------------------------------------------------------------------------------
+
+    private static class SproxydConfigLoaderCallable implements Callable<SproxydConfig> {
+
+        private final String filestoreID;
+        private final SproxydFileStorageFactory factory;
+
+        SproxydConfigLoaderCallable(String filestoreID, SproxydFileStorageFactory factory) {
+            super();
+            this.filestoreID = filestoreID;
+            this.factory = factory;
+        }
+
+        @Override
+        public SproxydConfig call() throws Exception {
+            return factory.doInitSproxydConfig(filestoreID);
+        }
+    }
+
 }
